'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mainWindow = exports.createWindow = undefined;

var _electron = require('electron');

var _electron2 = _interopRequireDefault(_electron);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _electronJsonStorage = require('electron-json-storage');

var _electronJsonStorage2 = _interopRequireDefault(_electronJsonStorage);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _jschardet = require('jschardet');

var _jschardet2 = _interopRequireDefault(_jschardet);

var _iconvLite = require('iconv-lite');

var _iconvLite2 = _interopRequireDefault(_iconvLite);

var _createMenu = require('./createMenu');

var _createMenu2 = _interopRequireDefault(_createMenu);

var _storeSetting = require('./storeSetting');

var _proVersion = require('./proVersion');

var _constValues = require('./constValues');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// createWindow.js

var mainWindow = void 0;

var iZoom = 10;

var curPosSize = { x: 10, y: 10, width: 600, height: 600, zoom: 10, home: 'https://workflowy.com' };
var prePosSize = { x: 10, y: 10, width: 600, height: 600, zoom: 10, home: 'https://workflowy.com' };
var flgMaxSize = false;

var sHomeUrl = _constValues.sHomeUrlBlank;

var menu = (0, _createMenu2.default)({ importFile: importFile,
  importFileToChild: importFileToChild,
  importFileToNote: importFileToNote,
  sendExport: sendExport,
  //sendExportFromWindow,
  sendGoHome: sendGoHome,
  sendGoForward: sendGoForward,
  sendGoBack: sendGoBack,
  sendGetUrl: sendGetUrl,
  sendSetBold: sendSetBold,
  sendSetItalic: sendSetItalic,
  sendSetUnderline: sendSetUnderline,
  sendStrLen: sendStrLen,
  sendZoom100: sendZoom100,
  sendZoomIn: sendZoomIn,
  sendZoomOut: sendZoomOut,
  sendSetHomeWF: sendSetHomeWF,
  sendSetHomeDL: sendSetHomeDL,
  sendViewVersion: sendViewVersion
});
var menu_home_wf = menu.getMenuItemById('set_home_workflowy');
var menu_home_dl = menu.getMenuItemById('set_home_dynalist');

function createWindow(option) {
  exports.mainWindow = mainWindow = new _electron.BrowserWindow({ width: 600, height: 600 });
  mainWindow.loadURL(_url2.default.format({
    pathname: _path2.default.join(__dirname, '../../index.html'),
    protocol: 'file:',
    slashes: true
  }));

  mainWindow.on('resize', function () {
    console.log('resize');
    setIniData();
  });

  mainWindow.on('maximize', function () {
    flgMaxSize = true;
    console.log('maximize');
  });

  mainWindow.on('unmaximize', function () {
    flgMaxSize = false;
    console.log('unmaximize');
  });

  mainWindow.on('close', function () {
    setIniDataSync();
  });

  mainWindow.on('closed', function () {
    exports.mainWindow = mainWindow = null;
  });

  _electron.Menu.setApplicationMenu(menu);

  getIniData();
}

function check_menu_home(sUrl) {
  if (sUrl === _constValues.sHomeUrlWorkflowy) {
    menu_home_wf.checked = true;
    menu_home_dl.checked = false;
  } else {
    menu_home_wf.checked = false;
    menu_home_dl.checked = true;
  }
}

function getIniData() {
  var obj = [600, 600];
  var pos = [10, 10];

  _electronJsonStorage2.default.get('window', function (error, data) {
    //if(error) throw error
    if (error) {
      console.log('cannot read ini-file.');
      return;
    }

    if (Object.keys(data).length === 0) {} else {
      if (data.x) {
        pos[0] = data.x;
      } else {
        console.log('err x');
      }
      if (data.y) {
        pos[1] = data.y;
      } else {
        console.log('err y');
      }
      if (data.width) {
        obj[0] = data.width;
      } else {
        console.log('err width');
      }
      if (data.height) {
        obj[1] = data.height;
      } else {
        console.log('err height');
      }
      if (data.zoom) {
        iZoom = data.zoom;
      } else {
        console.log('err zoom');
      }
      if (data.home) {
        sHomeUrl = data.home;
      } else {
        console.log('err home');
      }
      mainWindow.setPosition(pos[0], pos[1]);
      mainWindow.setSize(obj[0], obj[1]);
      prePosSize = JSON.parse(JSON.stringify(data));
      curPosSize = JSON.parse(JSON.stringify(data));
    }
  });

  if (sHomeUrl === _constValues.sHomeUrlBlank) {
    sHomeUrl = _constValues.sHomeUrlWorkflowy;
    check_menu_home(sHomeUrl);
    setIniData();
  }
}

_electron.ipcMain.on('SEND_DID_NAVIGATE', function (event, arg) {
  // 起動後初回のHomeURLを送信
  mainWindow.webContents.send('SET_HOME_URL', sHomeUrl);
  check_menu_home(sHomeUrl);
});

function setIniData() {
  // {"x":340,"y":192,"width":600,"height":600,"zoom":10}

  var json = { x: 10, y: 10, width: 600, height: 600, zoom: 10, home: 'https://workflowy.com' };

  prePosSize = JSON.parse(JSON.stringify(curPosSize));

  var size = mainWindow.getSize();
  console.log(size);

  var pos = mainWindow.getPosition();
  console.log(pos);

  json.x = pos[0];
  json.y = pos[1];
  json.width = size[0];
  json.height = size[1];
  json.zoom = iZoom;
  json.home = sHomeUrl;
  //console.log(json)
  curPosSize = JSON.parse(JSON.stringify(json));

  _electronJsonStorage2.default.set('window', json, function (error) {
    //console.log(error)
    //if(error) throw error
    if (error) {
      console.log('cannot write ini-file');
      return;
    }
  });
}

function setIniDataSync() {

  var json = { x: 10, y: 10, width: 600, height: 600, zoom: 10, home: 'https://workflowy.com' };

  var size = mainWindow.getSize();
  console.log(size);

  var pos = mainWindow.getPosition();
  console.log(pos);

  json.x = pos[0];
  json.y = pos[1];
  json.width = size[0];
  json.height = size[1];
  json.zoom = iZoom;
  json.home = sHomeUrl;
  //console.log(json)

  if (flgMaxSize) {
    json = JSON.parse(JSON.stringify(prePosSize));
  }

  (0, _storeSetting.setJsonDataSync)('window', json, function (error) {
    if (error) {
      console.log('cannot write ini-file');
      return;
    }
  });
}

function fileExist(filepath) {
  try {
    _fs2.default.statSync(filepath);
    return true;
  } catch (err) {
    return false;
  }
}

function readDataFile(callback) {
  _electron.dialog.showOpenDialog(mainWindow, {
    properties: ['openFile'],
    filters: [{ name: 'Text Files', extensions: ['txt'] }, { name: 'All Files', extensions: ['*'] }]
  }, function (filenames) {
    if (filenames !== undefined) {
      if (fileExist(filenames[0])) {
        _fs2.default.readFile(filenames[0], function (error, data) {
          //if(error) throw error
          if (error) {
            console.log("cannot read file.");
          }
          var detect = _jschardet2.default.detect(data);
          var sRet = _iconvLite2.default.decode(data, detect.encoding);
          var sFile = _path2.default.basename(filenames[0]);
          callback(sRet, sFile);
        });
      }
    }
  });
}

function importFile() {
  readDataFile(function (str, sFile) {
    _electron.clipboard.writeText(str);
    mainWindow.webContents.send('IMPORT_PASTE', 'paste');
  });
}

function importFileToChild() {
  readDataFile(function (str, sFile) {

    var chgTxt = '';
    var line = str.split(/\r\n|\r|\n/);

    chgTxt = '<?xml version=\"1.0\"?>\n<opml version=\"2.0\">\n<head><\/head><body>\n';
    chgTxt = chgTxt + '<outline text=\"' + sFile + '\" >\n';
    for (var ii = 1; ii < line.length; ii++) {
      line[ii] = line[ii].replace(/\r?\n/g, '');
      line[ii] = line[ii].replace(/&/g, '&amp;');
      line[ii] = line[ii].replace(/</g, '&lt;');
      line[ii] = line[ii].replace(/>/g, '&gt;');
      line[ii] = line[ii].replace(/"/g, '&quot;');
      line[ii] = line[ii].replace(/'/g, '&#39;');
      if (line[ii].length > 0) {
        chgTxt = chgTxt + '<outline text=\"' + line[ii] + '\" \/>\n';
      }
    }
    chgTxt = chgTxt + '<\/outline>\n<\/body><\/opml>';

    _electron.clipboard.writeText(chgTxt);
    mainWindow.webContents.send('IMPORT_PASTE', 'paste');
  });
}

function importFileToNote() {
  readDataFile(function (str, sFile) {
    var chgTxt = '';
    var line = str.split(/\r\n|\r|\n/);

    chgTxt = '<?xml version=\"1.0\"?>\n<opml version=\"2.0\">\n<head><\/head><body>\n';
    chgTxt = chgTxt + '<outline text=\"' + sFile + '\" _note=\"';
    for (var ii = 0; ii < line.length; ii++) {
      line[ii] = line[ii].replace(/\r?\n/g, '');
      line[ii] = line[ii].replace(/&/g, '&amp;');
      line[ii] = line[ii].replace(/</g, '&lt;');
      line[ii] = line[ii].replace(/>/g, '&gt;');
      line[ii] = line[ii].replace(/"/g, '&quot;');
      line[ii] = line[ii].replace(/'/g, '&#39;');
      chgTxt = chgTxt + line[ii] + '\&\#10\;';
    }
    chgTxt = chgTxt + '\"\/>\n<\/body><\/opml>';

    _electron.clipboard.writeText(chgTxt);
    mainWindow.webContents.send('IMPORT_PASTE', 'paste');
  });
}

_electron.ipcMain.on('SEND_RENDER_ZOOM', function (event, arg) {
  console.log("Zoom(main):" + arg);
  iZoom = arg;
  setIniData();
});

function saveDialog(sFileName, sText) {

  _electron.dialog.showSaveDialog(mainWindow, { defaultPath: sFileName,
    filters: [{ name: 'Text', extensions: ['txt'] }]
  }, function (filename) {
    if (filename != undefined) {
      _fs2.default.writeFile(filename, sText, function (error) {
        //if(error) throw error
        if (error) {
          console.log('saveDialog error!');
        }
      });
    }
  });
}

_electron.ipcMain.on('EXPORT_RETURN', function (event, arg) {
  console.log(arg);

  var line = arg.split(/\r\n|\r|\n/);
  var chgTxt = line.join('\n');
  saveDialog(line[0], chgTxt);
});

// ipcMain.on('EXPORT_RETURN', function(event, arg){
//
//   console.log(arg)
//
//   let line = arg.split(/\r\n|\r|\n/)
//
//   let chgTxt = ''
//   let strTmp = ''
//   let flgStart = true
//
//   for( let ii=0;ii<line.length-1;ii++){
//     line[ii] = line[ii].replace(/<\/div>/g,'')
//       .replace(/<span .+?>/g, '')
//       .replace(/<\/span>/g, '\n\tspan')
//       .replace(/<div class=\"([0-9a-zA-Z\_\- ]+)\"[0-9a-zA-Z\-\_\= \"\:\;\(\)\,\.]*>/g,"\t$1\n")
//       .replace(/<div>/g,'')
//       .replace(/<\/a>/g,'')
//       .replace(/<a .+?>/g,'')
//       .replace(/<\/h.>/g,'')
//       .replace(/<h. .+?>/g,'')
//
//     if(/\n$/.test(line[ii])){
//       line[ii] = line[ii].replace(/\n$/, '')
//     }
//
//     if( /\t.*selected ?/.test(line[ii]) ){
//       flgStart = false
//     }
//
//     strTmp = strTmp + line[ii] + "\n"
//   }
//
//   let line2 = strTmp.split(/\n/)
//   for( let ii=0;ii<line2.length;ii++){
//     line2[ii] = line2[ii].replace(/&lt;/g,'<')
//       .replace(/&gt;/g,'>')
//       .replace(/&amp;/g,'&')
//       .replace(/&quot;/g, '"')
//       .replace(/&nbsp;/g, ' ')
//     if( /^\t.*selected ?/.test(line2[ii]) ){
//       flgStart = true
//     }else if( /^\t/.test(line2[ii]) ){
//       //// 180515 start /////
//       if( /^\tspan\t[a-z]/.test(line2[ii]) ){
// 	// (^^;
//       }else if( /^\tspan/.test(line2[ii]) ){
// 	let str = line2[ii].replace(/^\tspan/,'')
// 	chgTxt = chgTxt.replace(/\s$/,'') + str + "\n"
//       }else{
// 	// (^^;
//       }
//       //// 180515 end /////
//     }else{
//       if(flgStart === true){
// 	chgTxt = chgTxt + line2[ii] + "\n"
//       }
//     }
//
//   }
//
//   // ファイル名抽出のためだけにsplit
//   let line3 = chgTxt.split(/\n/)
//
//   saveDialog( line3[0], chgTxt )
//
// })

// function exportFormatTextToPlanText ( arg ){
//   let line = arg.split(/\r\n|\r|\n/)
//
//   let chgTxt = ''
//   let strTmp = ''
//  
//   for( let ii=0;ii<line.length;ii++) {
//     strTmp = line[ii].replace(/^\s*\-\s+/,'').replace(/^\s*\*\s+/,'')
//     chgTxt = chgTxt + strTmp + '\n'
//   }
//
//   saveDialog( line[0], chgTxt )
// }

//function exportPlanTextToPlanText ( arg ){
//  let line = arg.split(/\r\n|\r|\n/)
//  let chgTxt = line.join('\n')
//  saveDialog( line[0], chgTxt )
//}

// function exportOpmlToPlanText ( arg ){
//   console.log(arg)
//  
//   let line = arg.split(/\r\n|\r|\n/)
//  
//   let chgTxt = ''
//   let strTmp = ''
//  
//   for( let ii=0;ii<line.length;ii++) {
//     if( /^\s+<outline\s/.test(line[ii]) ){
//       if( /\s_note\s*\=\s*\"/.test(line[ii]) ){
// 	if( line[ii].match(/text\s*\=\s*\"(.+)\".+_note\s*\=\s*\"(.+)\"/) ){
// 	  let strTmp1 = line[ii].match(/text\s*\=\s*\"(.+)\".+_note\s*\=\s*\"(.+)\"/)
// 	  //console.log( strTmp1 )	// match配列の0番目は全体が入るらしい
// 	  strTmp1[1] = strTmp1[1].replace(/\<\/?[bui]\>/g,'')
// 	  strTmp = strTmp + strTmp1[1] + '\n' + strTmp1[2] + '\n'
// 	}
//       }else{
// 	if( line[ii].match(/text\s*\=\s*\"(.+)\"/) ) {
// 	  let strTmp2 = line[ii].match(/text\s*\=\s*\"(.+)\"/)
// 	  //console.log( strTmp2 )	// match配列の0番目は全体が入るらしい
// 	  strTmp2[1] = strTmp2[1].replace(/\<\/?[bui]\>/g,'')
// 	  strTmp = strTmp + strTmp2[1] + '\n'
// 	}
//       }
//     }
//   } 
//  
//   let line2 = strTmp.split(/\n/)
//   for( let ii=0;ii<line2.length;ii++){
//     line2[ii] = line2[ii].replace(/&lt;/g,'<')
//       .replace(/&gt;/g,'>')
//       .replace(/&amp;/g,'&')
//       .replace(/&quot;/g, '"')
//       .replace(/&nbsp;/g, ' ')
//       .replace(/&#10;/g, '\n')
//     chgTxt = chgTxt + line2[ii] + '\n'
//   }
//  
//   // ファイル名抽出のためだけにsplit
//   let line3 = chgTxt.split(/\n/)
//  
//   saveDialog( line3[0], chgTxt )
//  
// }

//ipcMain.on('EXPORT_FROM_WINDOW_RETURN', function(event, arg){
////  if(/^<\?xml/.test(arg)){
////    exportOpmlToPlanText(arg)
////  }else{
//    exportPlanTextToPlanText(arg)
////  }
//})


function sendExport() {
  var arg = 'export-wf';
  if (sHomeUrl == _constValues.sHomeUrlDynalist) {
    arg = 'export-dl';
  }
  mainWindow.webContents.send('EXPORT', arg);
  //console.log("Export")
}

//function sendExportFromWindow(){
//    mainWindow.webContents.send('EXPORT_FROM_WINDOW', 'Export')
//  //console.log("Export")
//}

//function sendExportFromWindowPlanText(){
//  mainWindow.webContents.send('EXPORT_FROM_WINDOW', 'ExportPlanText')
//  //console.log("Export")
//}

function sendGoHome() {
  mainWindow.webContents.send('GO_HOME', "go home");
  //console.log("go home")
}

function sendGoForward() {
  mainWindow.webContents.send('GO_FORWARD', "forward");
  //console.log("forward")
}

function sendGoBack() {
  mainWindow.webContents.send('GO_BACK', "back");
  ///console.log("back")
}

function sendGetUrl() {
  mainWindow.webContents.send('GET_URL', "get url");
  //console.log("get url")
}

function sendSetBold() {
  mainWindow.webContents.send('SET_BOLD', "set bold");
}

function sendSetItalic() {
  mainWindow.webContents.send('SET_ITALIC', "set italic");
}

function sendSetUnderline() {
  mainWindow.webContents.send('SET_UNDERLINE', "set underline");
}

function sendStrLen() {
  var arg = 'export-wf';
  if (sHomeUrl == _constValues.sHomeUrlDynalist) {
    arg = 'export-dl';
  }
  mainWindow.webContents.send('STR_LEN', arg);
}

function sendZoom100() {
  iZoom = 10;
  mainWindow.webContents.send('ZOOM_100', iZoom);
}

function sendZoomIn() {
  mainWindow.webContents.send('ZOOM_IN', iZoom);
}

function sendZoomOut() {
  mainWindow.webContents.send('ZOOM_OUT', iZoom);
}

function setHome(sUrl) {
  if (sHomeUrl != sUrl) {
    sHomeUrl = sUrl;
    check_menu_home(sHomeUrl);
    mainWindow.webContents.send('SET_HOME_URL', sHomeUrl);
    setIniData();
  } else {
    check_menu_home(sHomeUrl);
  }
}

function sendSetHomeWF() {
  setHome(_constValues.sHomeUrlWorkflowy);
}

function sendSetHomeDL() {
  setHome(_constValues.sHomeUrlDynalist);
}

function sendViewVersion() {
  mainWindow.webContents.send('VIEW_VERSION', _proVersion.proVersion);
  console.log('view version');
}

exports.createWindow = createWindow;
exports.mainWindow = mainWindow;